/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.common;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.common.AbstractProcessor;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.themes.ThemeDocument;

public final class FlowDepthDischargeProcessor extends AbstractProcessor {

    /* discharge */
    public static final String FACET_FLOW_DEPTH_DISCHARGE = "sinfo_facet_flow_depth_discharge";
    public static final String FACET_FLOW_DEPTH_DISCHARGE_Y_AXIS_LABEL = "common.export.csv.header.q";
    public static final String FACET_FLOW_DEPTH_DISCHARGE_DESCRIPTION = "facet.flow_depth.discharge";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_FLOW_DEPTH_DISCHARGE);
    }

    public FlowDepthDischargeProcessor() {
        super(FACET_FLOW_DEPTH_DISCHARGE_Y_AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    protected SInfoResultType doGetType(final String facetName) {

        if (FACET_FLOW_DEPTH_DISCHARGE.contentEquals(facetName))
            return SInfoResultType.discharge;

        final String error = String.format("Unknown facet name: %s", facetName);
        throw new UnsupportedOperationException(error);
    }

    public static Facet createFlowDepthDischargeFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int index, final String wstLabel) {
        final CallMeta meta = context.getMeta();
        final String yAxisLabel = Resources.getMsg(meta, FACET_FLOW_DEPTH_DISCHARGE_Y_AXIS_LABEL, FACET_FLOW_DEPTH_DISCHARGE_Y_AXIS_LABEL);
        return AbstractProcessor.createFacet(context, hash, id, result, index, yAxisLabel, FACET_FLOW_DEPTH_DISCHARGE, FACET_FLOW_DEPTH_DISCHARGE_DESCRIPTION,
                wstLabel);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        return buildSeriesForType(generator, bundle, theme, visible, doGetType(bundle.getFacetName()), GAP_DISTANCE);
    }
}